//-------------------------------------------------------------------------------
// <copyright file="EventSettings.h" company="LiquidRobotics">
//
// Copyright (c) 2014 Liquid Robotics, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// This software is associated with a release.  Liquid Robotics makes no
// representation that the design, interfaces or component distribution will
// remain unchanged in future releases.
//
// </copyright>
//
// <summary>
//   Declaration of the EventSettings class, a serializable entity that holds the
//   name and type settings that identify an event.
// </summary>
//
// <owner>Mike Cookson</owner>
//-------------------------------------------------------------------------------

#ifndef __EVENTLIB_EVENTSETTINGS_H__
#define __EVENTLIB_EVENTSETTINGS_H__

#include "EventType.h"
#include "LqrSMACSharedLib.h"
#include <ostream>
#include <string>

namespace LiquidRobotics {
namespace EventLib {

class SHARED_LIB_PUBLIC EventSettings
{
public:

	typedef std::string string;

	EventSettings();
	EventSettings(const string& _eventName, const string& _eventTypeName, const string& _eventDomainName);
	EventSettings(const string& _eventName, const EventType& _eventType);
	EventSettings(const EventSettings& other);

	EventSettings& operator=(const EventSettings& other);
	bool operator==(const EventSettings& other) const;
	bool operator!=(const EventSettings& other) const;

	bool Empty() const;

	const string& GetName() const;
	void SetName(const string& newValue);

	const string& GetTypeName() const;
	void SetTypeName(const string& newValue);

	const string& GetDomainName() const;
	void SetDomainName(const string& newValue);

	const EventType& GetType() const;
	void SetType(const EventType& newValue);

	void Load(const string& fileName);
	void Save(const string& fileName);

private:

	friend class boost::serialization::access;

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & BOOST_SERIALIZATION_NVP(eventName);
		ar & BOOST_SERIALIZATION_NVP(eventType);
	}

	string eventName;
	EventType eventType;
};

std::ostream& operator<<(std::ostream& stm, const EventSettings& settings);

}
}

#endif
